using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WindowsApplication1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            icImagingControl1.ShowDeviceSettingsDialog();

            if (!icImagingControl1.DeviceValid)
            {
                cmdStartLive.Enabled = false;
                cmdSettings.Enabled = false;
            }

            cmdStopLive.Enabled = false;
            icImagingControl1.MemoryCurrentGrabberColorformat = TIS.Imaging.ICImagingControlColorformats.ICRGB24;
            icImagingControl1.LiveCaptureContinuous = true;
            icImagingControl1.OverlayBitmap.Enable = true;
        }
        private void cmdStartLive_Click(object sender, EventArgs e)
        {
            icImagingControl1.LiveStart();
            icImagingControl1.OverlayBitmap.DrawLine(Color.Blue, 0, icImagingControl1.LiveDisplayHeight / 2, icImagingControl1.LiveDisplayWidth, icImagingControl1.LiveDisplayHeight / 2);
            cmdStartLive.Enabled = false;
            cmdStopLive.Enabled = true;
        }

        private void cmdStopLive_Click(object sender, EventArgs e)
        {
            cmdStartLive.Enabled = true;
            cmdStopLive.Enabled = false;
            icImagingControl1.LiveStop();
        }

        private void cmdSettings_Click(object sender, EventArgs e)
        {
            icImagingControl1.ShowPropertyDialog();
        }

        private void cmdDevice_Click(object sender, EventArgs e)
        {
            if (icImagingControl1.LiveVideoRunning)
            {
                icImagingControl1.LiveStop();
            }

            icImagingControl1.ShowDeviceSettingsDialog();

            if (icImagingControl1.DeviceValid)
            {
                cmdStartLive.Enabled = true;
                cmdSettings.Enabled = true;
            }
        }
        private bool findLaserPoint(out int xLaser, out int yLaser)
        {
            TIS.Imaging.ImageBuffer imagebuffer = icImagingControl1.ImageActiveBuffer;
            int redPixel = 0;
            xLaser = 0;
            yLaser = 0;

            // Check each pixel in the image if the red value is greater than the desired threshold (230).


            for (int y = 0; y < imagebuffer.Lines; y++)
            {
                for (int x = 0; x < imagebuffer.PixelPerLine; x++)
                {
                    // x and y index for the imagebuffer.
                    int indexXred = x * 3 + 2;
                    int indexY = imagebuffer.Lines - y - 1;

                    if (imagebuffer[indexXred, indexY] > 230)
                    {
                        // Count red pixels.
                        redPixel++;

                        xLaser = xLaser + x;
                        yLaser = yLaser + y;
                    }
                }
            }
            // Calculate the center of all red pixels.
            if (redPixel > 0)
            {
                xLaser = xLaser / redPixel;
                yLaser = yLaser / redPixel;

                return true;
            }
            else
            {
                return false;
            }
        }
        private void UpdateControlText(Control ctrl, string text)
        {
            ctrl.Text = text;
        }

        private delegate void UpdateControlTextDelegate(Control ctrl, string text);

        private void SetControlText(Control ctrl, string text)
        {
           BeginInvoke(new UpdateControlTextDelegate(UpdateControlText), new object[] { ctrl, text });
        }

        private void icImagingControl1_ImageAvailable(object sender, TIS.Imaging.ICImagingControl.ImageAvailableEventArgs e)
        {
            int x, y;
            double angle, angleradians;
            double range;
            double h = 102;

            if (chkRange.Checked)
            {
                if (findLaserPoint(out x, out y))
                {
                    // Calculate the angle.
                    angle = 0.0545454545 * x + 52.2890909091;
                    // Convert degrees to radians.
                    angleradians = (Math.PI/180) * angle;
                    // Calculate the range Tan(angle) * h.
                    range = Math.Round(Math.Tan(angleradians) * h, 0);
                    // Update the textboxes.
                    SetControlText(txtX, x.ToString());
                    SetControlText(txtY, y.ToString());
                    SetControlText(txtAngle, Math.Round(angle,2).ToString());
                    SetControlText(txtRange, range.ToString() + " mm");
                }
                else
                {
                    SetControlText(txtX,"No laser");
                    SetControlText(txtY, "No laser");
                }
            }
        }
    }
}
